import cachetools.func
import radicale.syno.constant as constant
import radicale.syno.permission as permission
import radicale.syno.vobject_wrapper.utils as vobject_utils
import vobject
from . import base as models
from pwd import getpwnam
from radicale.syno import utils
from radicale.syno.lib import SynoLib
from radicale.syno.singleton import Singleton
from radicale.storage import get_uid_from_object
from radicale.syno.vobject_wrapper.person import Person
from radicale.xmlutils import find_tag_and_time_range
from sqlalchemy import Sequence, create_engine
from sqlalchemy.sql import func
from sqlalchemy.orm import scoped_session, sessionmaker

vobject.vcard.wacky_apple_photo_serialize = False


def get_collection_model(collection_type):
    """Get collection model from ``collection_type``
    """
    if collection_type not in [
        constant.TYPE_PRINCIPAL,
        constant.TYPE_ADDRESSBOOK
    ]:
        return None

    if collection_type == constant.TYPE_PRINCIPAL:
        model_collection = models.Principal
    elif collection_type == constant.TYPE_ADDRESSBOOK:
        model_collection = models.Addressbook
    return model_collection


class Model(metaclass=Singleton):

    def __init__(self, db_url, logger):
        engine = create_engine(db_url)
        session_factory = sessionmaker(bind=engine)
        session = scoped_session(session_factory)
        models.BaseModel.set_session(session)
        self._session = session
        self._logger = logger
        self._lib = SynoLib()

    def commit(self):
        """Try to commit, rollback when commit failed"""
        try:
            self._session.commit()
        except Exception as e:
            self._session.rollback()
            self._logger.error("Failed to commit: {0}".format(e))

    def acquire_lock(self):
        """Acquire lock.
        """
        session = self._session()
        # XXX: should we add timeout setting here?
        while session.info.get("lock", False):
            pass
        session.info["lock"] = True

    def release_lock(self):
        """Release lock.
        """
        session = self._session()
        session.info["lock"] = False

    def cleanup(self):
        """Clean up.
        """
        self._logger.debug("Cleaning session up.")
        self._session.remove()

    def get_owner_id(self, principal_uri):
        """Get owner_id corresponding to ``principal_uri``

        Return None when ``principal_uri`` is not set or encounter any error.
        """
        if not principal_uri:
            return None
        try:
            user_name = utils.principal_uri_to_user_name(principal_uri)
            user_real_name = self._lib.get_real_name(user_name)
            owner_id = getpwnam(user_real_name).pw_uid
        except BaseException:
            self._logger.error(
                "Failed to getpwnam for principal_uri [{0}], user_real_name [{1}]"
                .format(principal_uri, user_real_name)
            )
            return None
        return owner_id

    def _get_next_seq_id(self, seq_name):
        session = self._session()
        next_id = session.execute(Sequence(seq_name))
        return next_id

    def get_next_addressbook_id(self):
        return self._get_next_seq_id("addressbook_id_seq")

    def get_principal_id(self, principal_uri):
        """Get principal id from ``principal_uri``.

        Return None when ``principal_uri`` is not set or encounter any error.
        """
        if not principal_uri:
            return None

        owner_id = self.get_owner_id(principal_uri)

        principal = models.Principal.where(
            owner_id=owner_id,
            type=constant.PRINCIPAL_TYPE_USER
        ).first()
        return principal.id if principal else None

    def get_addressbook_id(self, addressbook_uri):
        """Get addressbook id from ``addressbook_uri``.

        Return value: id, None when ``addressbook_uri`` is not set or is not exists.
        """
        if not addressbook_uri:
            return None

        addressbook = models.Addressbook.where(uri=addressbook_uri).first()
        if addressbook:
            return addressbook.id
        return None

    def is_ro_addressbook(self, collection_uri):
        """Determine if ``collection_uri`` belongs to RO addressbook.
        """
        if not collection_uri:
            return False

        ro_addressbook = models.Addressbook.where(
            uri=collection_uri,
            type__in=[
                constant.ADDRESSBOOK_TYPE_SUGGESTED,
                constant.ADDRESSBOOK_TYPE_DEFAULT_PUBLIC,
                constant.ADDRESSBOOK_TYPE_EXTERNAL_SOURCE
            ]
        ).first()
        return True if ro_addressbook else False

    @cachetools.func.ttl_cache(maxsize=128, ttl=30)
    def get_privilege(self, user, path):
        """Get collection(principal, addressbook) privilege regard to ``user``.

        Return {PRIV_OWN | PRIV_(GROUP)_MANAGE | PRIV_(GROUP)_WRITE | PRIV_(GROUP)_READ | None}
        """
        if bool(user) and (not path or path == constant.PATH_ROOT_COLLECTION):
            return constant.PRIV_READ

        principal_uri, addressbook_uri = utils.get_principal_uri_and_addressbook_uri(
            path
        )

        target_principal_id = self.get_principal_id(principal_uri)
        addressbook_id = self.get_addressbook_id(addressbook_uri)

        # path leads to principal
        if not addressbook_id:
            if target_principal_id:
                record = models.Principal.where(id=target_principal_id).first()

                owner_id = self.get_owner_id(user)
                return constant.PRIV_OWN if owner_id == record.owner_id else None
            else:
                return None

        login_principal_id = self.get_principal_id(user)

        records = (
            self._session.query(
                models.t_principal_id_to_addressbook_privilege_view
            ).filter_by(
                id_principal=login_principal_id,
                id_addressbook=addressbook_id
            ).all()
        )

        priv = None
        for record in records:
            if priv is None or permission.is_greater(record.mode, priv):
                priv = record.mode

        # Reserved addressbooks(TeamContacts, Suggested and ExternalSource) are
        # not allowed to modify via CardDAV protocol even for users with
        # priviledge above READ_ONLY. Threfore, here reset addressbook
        # permission back to READ_ONLY.
        if permission.is_greater(priv, constant.PRIV_READ) and self.is_ro_addressbook(addressbook_uri):
            return constant.PRIV_READ

        return priv

    def get_addressbook_objects(self, addressbook_id, addressbook_object_uris):
        """Get addressbook objects corresponding to ``addressbook_object_uris`` and belong to ``addressbook_id``.

        Yield href, props.
        """
        if not addressbook_id:
            self._logger.error(
                """
                Failed to get addressbook object:
                addressbook_id: {0}, addressbook_object_uris: {1}
                """.format(addressbook_id,
                           addressbook_object_uris)
            )
            return

        records = models.AddressbookObject.where(
            uri__in=addressbook_object_uris,
            id_addressbook=addressbook_id,
            is_hidden=False
        ).all()

        for record in records:
            try:
                vobject_item = vobject.readOne(record.vcard_text)
            except Exception as e:
                # <Contacts> #951
                # ignore invalid vcard content so that radicale
                # doesn't abort processing remaining record.
                self._logger.error(
                    """
                    Failed to load item {0} in addressbook_id {1}:
                    {2}
                    """.format(record.uri,
                               addressbook_id,
                               e)
                )
                continue

            tag, _, _ = find_tag_and_time_range(vobject_item)

            item = vobject_item
            href = record.uri
            last_modified = utils.get_http_date(record.modified_time)
            text = record.vcard_text
            etag = '"{0}"'.format(record.md5)  # Encoded as quoted-string (see RFC 2616).
            uid = get_uid_from_object(vobject_item)
            name = vobject_item.name
            component_name = tag

            props = {
                "item": item,
                "href": href,
                "last_modified": last_modified,
                "text": text,
                "etag": etag,
                "uid": uid,
                "name": name,
                "component_name": component_name
            }
            yield href, props

    def list_addressbooks(self, principal_id):
        """
        List addressbooks corresponding to ``principal_id``.

        Yield addressbook uris.
        """
        if principal_id is None:
            return

        mappings = models.ManyPrincipalHasManyAddressbook.where(
            id_principal=principal_id,
            status=constant.STATUS_ACCEPTED
        ).all()

        addressbooks = models.Addressbook.where(
            id__in=[mapping.id_addressbook for mapping in mappings]
        )

        self._logger.debug(
            """
            addressbooks belongs to principal_id {0}: {1}
            """.format(principal_id,
                       addressbooks)
        )

        for addressbook in addressbooks:
            yield addressbook.uri

    def list_addressbook_objects(self, addressbook_id):
        """
        List addressbook objects corresponding to ``addressbook_id``.

        Yield addressbook object uris.
        """
        if not addressbook_id:
            return

        self._logger.debug(
            """
            Listing addressbook objects for addressbook_id: {0}
            """.format(addressbook_id)
        )

        addressbook_objects = models.AddressbookObject.where(
            id_addressbook=addressbook_id,
            is_hidden=False
        ).all()

        self._logger.debug(
            "Addressbook objects: {0}".format(addressbook_objects)
        )

        for abo in addressbook_objects:
            # <Contacts> #951 #984
            # avoid return contact containing invalid vcard content.
            # Otherwise, the latter processing would abort execution
            try:
                vobject_item = vobject.readOne(abo.vcard_text)
            except Exception as e:
                self._logger.error(
                    """
                    Failed to load item {0} in addressbook_id {1}:
                    {2}
                    """.format(abo.uri,
                               addressbook_id,
                               e)
                )
                continue
            yield abo.uri

    def _make_sure_principal_exists(self, principal_uri):
        """Create principal if not exists.

        ``principal_uri`` must not be empty.

        Return principal_id.
        """
        if not principal_uri:
            self._logger.debug(
                "Insufficient information to _make_sure_principal_exists. principal_uri: {0}"
                .format(principal_uri)
            )
            return

        principal_id = self.get_principal_id(principal_uri)

        if not principal_id:
            owner_id = self.get_owner_id(principal_uri)
            principal = models.Principal.create(
                owner_id=owner_id,
                type=constant.PRINCIPAL_TYPE_USER,
                display_name=principal_uri,
                account_name=principal_uri
            )
            principal_id = principal.id

            self._logger.debug(
                "Create principal: principal_id: {0}, uri: {1}, owner_id: {2}"
                .format(principal_id,
                        principal_uri,
                        owner_id)
            )

        return principal_id

    def _make_sure_addressbook_exists(self, addressbook_uri):
        """Make sure ``addressbook_uri`` exists . Create if not exists.

        ``addressbook_uri`` must not be empty.

        Return addressbook_id.
        """
        if not addressbook_uri:
            self._logger.debug(
                """
                Insufficient information to make sure addressbook exists.
                addressbook_uri: {0}
                """.format(addressbook_uri)
            )
            return

        addressbook_id = self.get_addressbook_id(addressbook_uri)

        if not addressbook_id:
            addressbook_id = self.get_next_addressbook_id()
            addressbook = {
                "id": addressbook_id,
                "uri": addressbook_uri,
                "display_name": "Default display name",
                "description": "Should be modified later",
                "color": "0xFFFFFF"
            }

            self._logger.debug(
                """
                Created addressbook:
                    addressbook_id: {0}, addressbook_uri: {1}
                """.format(
                    addressbook_id,
                    addressbook_uri,
                )
            )

            models.Addressbook.create(**addressbook)

        return addressbook_id

    def _set_privilege(self, principal_id, addressbook_id, privilege):
        """
        Set privilege.

        ``principal_id``, ``addressbook_id`` and ``privilege`` must not be empty.
        """
        if not principal_id or not addressbook_id or privilege not in [
            constant.PRIV_OWN,
            constant.PRIV_MANAGE,
            constant.PRIV_WRITE,
            constant.PRIV_READ
        ]:
            self._logger.debug(
                """
                Insufficient information to _set_privilege.
                principal_uri: {0},
                collection_uri: {1},
                privilege: {2}.
                """.format(principal_id,
                           addressbook_id,
                           privilege)
            )
            return

        self._logger.debug(
            """
            Set privilege: {0}, principal_id: {1}, addressbook_id: {2}.
            """.format(privilege,
                       principal_id,
                       addressbook_id)
        )

        models.ManyPrincipalHasManyAddressbook.create(
            id_principal=principal_id,
            id_addressbook=addressbook_id,
            mode=privilege,
            status=constant.STATUS_ACCEPTED
        )

    def make_sure_path_exists(self, path, collection_type):
        """Make sure path exists.

        ``path`` must not be empty, should already be sanitized.

        ``collection_type`` must be in [TYPE_PRINCIPAL, TYPE_ADDRESSBOOK]
        """
        principal_uri, addressbook_uri = utils.get_principal_uri_and_addressbook_uri(
            path)

        if collection_type == constant.TYPE_PRINCIPAL:
            self._make_sure_principal_exists(principal_uri)
        elif collection_type == constant.TYPE_ADDRESSBOOK:
            principal_id = self._make_sure_principal_exists(principal_uri)
            addressbook_id = self._make_sure_addressbook_exists(addressbook_uri)
            if not self.get_privilege(principal_uri, path):
                self._set_privilege(
                    principal_id,
                    addressbook_id,
                    constant.PRIV_OWN
                )

    def set_addressbook_meta(
        self,
        addressbook_id,
        display_name,
        color,
        description
    ):
        """Set metadata values for addressbook.

        ``addressbook_id`` and ``display_name`` must not be empty.

        """
        if not addressbook_id or not display_name:
            self._logger.error(
                """
                Insufficient information to set_addressbook_meta:
                addressbook_id: {0}
                display_name: {1},
                color: {2},
                description: {3}
                """.format(addressbook_id,
                           display_name,
                           color,
                           description)
            )
            return

        record = models.Addressbook.where(id=addressbook_id).first()

        if record:
            setattr(record, 'display_name', display_name)
            setattr(record, 'color', color)
            setattr(record, 'description', description)

    def upload_addressbook_object(
        self,
        principal_id,
        addressbook_id,
        href,
        vobject_item
    ):
        """Upload addressbook object.

        ``principal_id``, ``addressbook_id``, ``href`` and ``vobject_item`` must not be empty.
        """
        if not principal_id or not addressbook_id or not href or not vobject_item:
            self._logger.error(
                """
                Can't upload_vobject_item:
                principal_id: {0},
                addressbook_id: {1},
                href: {2},
                vobject_item: {3}
                """.format(principal_id,
                           addressbook_id,
                           href,
                           vobject_item)
            )
            return

        href_suffix = constant.VCF_SUFFIX

        try:
            uid = href.split(href_suffix)[0]
            utils.set_uid_to_object(uid, vobject_item)
            text = vobject_item.serialize()
        except Exception as e:
            self._logger.error(
                """
                Error occurs while uploading addressbook_object {0} to addressbook {1}:
                {2}
                """.format(href,
                           addressbook_id,
                           e)
            )
        else:
            if vobject_utils.get_props_first_val(
                vobject_item,
                constant.VCARD_PROPS_KIND
            ) == "group":
                kind = constant.ADDRESSBOOK_OBJECT_KIND_GROUP
            else:
                kind = constant.ADDRESSBOOK_OBJECT_KIND_PERSON

            uri = href
            addressbook_object = {
                "id_addressbook": addressbook_id,
                "uri": uri,
                "vcard_text": text,
                "kind": kind
            }

            addressbook_object_revision = {
                "id_principal": principal_id,
                "id_addressbook": addressbook_id,
                "uri": uri,
                "is_deleted": False
            }

            model_object = models.AddressbookObject
            model_revision = models.AddressbookObjectRevision

            # update or create addressbook_object
            db_addressbook_object = model_object.where(
                uri=addressbook_object.get("uri")
            ).first()

            if db_addressbook_object:
                db_addressbook_object.update(**addressbook_object)
            else:
                db_addressbook_object = model_object.create(
                    **addressbook_object
                )
            if kind == constant.ADDRESSBOOK_OBJECT_KIND_PERSON:
                person = Person(vobject_item)
                self.update_addressbook_object_metadata(
                    db_addressbook_object.id,
                    person
                )
                self.update_addressbook_object_search_token(
                    db_addressbook_object.id,
                    person
                )

            # create addressbook_object_revision
            model_revision.create(**addressbook_object_revision)

    def update_addressbook_object_search_token(self, abo_id, person):

        def append_insert_cmd(cmds, token, type):
            if not isinstance(cmds, (list, )) or not token or not type:
                return
            cmds.append(
                model.insert().values(
                    id_addressbook_object=abo_id,
                    token=token,
                    type=type
                )
            )

        def append_insert_cmd_list(cmds, token_list, type):
            if not isinstance(cmds,
                              (list,
                               )) or not isinstance(token_list,
                                                    (list,
                                                     )) or not type:
                return
            for token in token_list:
                append_insert_cmd(cmds, token, type)

        model = models.t_addressbook_object_search_token
        cmds = []

        # delete original records first
        cmds.append(model.delete(model.c.id_addressbook_object == abo_id))

        append_insert_cmd(
            cmds,
            person.full_name,
            constant.ABO_SEARCH_TOKEN_TYPE_FULL_NAME
        )

        append_insert_cmd(
            cmds,
            person.given_name,
            constant.ABO_SEARCH_TOKEN_TYPE_GIVEN_NAME
        )
        append_insert_cmd(
            cmds,
            person.middle_name,
            constant.ABO_SEARCH_TOKEN_TYPE_MIDDLE_NAME
        )
        append_insert_cmd(
            cmds,
            person.family_name,
            constant.ABO_SEARCH_TOKEN_TYPE_FAMILY_NAME
        )
        append_insert_cmd(
            cmds,
            person.title,
            constant.ABO_SEARCH_TOKEN_TYPE_JOB_TITLE
        )
        append_insert_cmd(
            cmds,
            person.department,
            constant.ABO_SEARCH_TOKEN_TYPE_DEPARTMENT
        )
        append_insert_cmd(
            cmds,
            person.company,
            constant.ABO_SEARCH_TOKEN_TYPE_COMPANY
        )
        append_insert_cmd_list(
            cmds,
            person.emails,
            constant.ABO_SEARCH_TOKEN_TYPE_EMAIL
        )
        append_insert_cmd_list(
            cmds,
            person.telephones,
            constant.ABO_SEARCH_TOKEN_TYPE_TEL
        )
        append_insert_cmd_list(
            cmds,
            person.addresses,
            constant.ABO_SEARCH_TOKEN_TYPE_ADR
        )
        append_insert_cmd_list(
            cmds,
            person.urls,
            constant.ABO_SEARCH_TOKEN_TYPE_URL
        )
        append_insert_cmd_list(
            cmds,
            person.dates,
            constant.ABO_SEARCH_TOKEN_TYPE_DATE
        )
        append_insert_cmd_list(
            cmds,
            person.ims,
            constant.ABO_SEARCH_TOKEN_TYPE_IM
        )
        append_insert_cmd(
            cmds,
            person.phonetic_given_name,
            constant.ABO_SEARCH_TOKEN_TYPE_PHONETIC_GIVEN_NAME
        )
        append_insert_cmd(
            cmds,
            person.phonetic_middle_name,
            constant.ABO_SEARCH_TOKEN_TYPE_PHONETIC_MIDDLE_NAME
        )
        append_insert_cmd(
            cmds,
            person.phonetic_family_name,
            constant.ABO_SEARCH_TOKEN_TYPE_PHONETIC_FAMILY_NAME
        )
        append_insert_cmd(
            cmds,
            person.nickname,
            constant.ABO_SEARCH_TOKEN_TYPE_NICKNAME
        )
        append_insert_cmd(
            cmds,
            person.note,
            constant.ABO_SEARCH_TOKEN_TYPE_NOTE
        )

        for cmd in cmds:
            self._session.execute(cmd)

    def update_addressbook_object_metadata(self, abo_id, person):
        """
        Update addressbook object metadata if kind != group
        """
        record = {
            "id_addressbook_object": abo_id,
            "full_name": person.full_name,
            "company": person.company,
            "department": person.department,
            "job_title": person.title,
            "email": person.primary_email,
            "telephone": person.primary_telephone,
            "date": person.primary_date,
            "birthday": person.birthday,
            "photo": person.photo
        }

        # update or create record_object
        db_record = models.AddressbookObjectMetadata.where(
            id_addressbook_object=abo_id
        ).first()

        if db_record:
            db_record.update(**record)
        else:
            db_record = models.AddressbookObjectMetadata.create(**record)

    def delete_all_addressbook_objects(self, addressbook_id):
        """Delete all addressbook objects in ``addressbook_id``.

        ``addressbook_id`` must not be empty.
        """
        if not addressbook_id:
            self._logger.error("Can't delete_addressbook_objects ")
            return

        model_object = models.AddressbookObject

        records = model_object.where(id_addressbook=addressbook_id).all()
        record_ids = ()
        for r in records:
            record_ids += (r.id, )
        model_object.destroy(*record_ids)

    def get_max_revision_num(self, addressbook_id):
        """Get max revision number for collection.

        ``addressbook_id`` must not be empty.

        Return 0 if there is no record addressbook_object_revision table
        """
        if not addressbook_id:
            self._logger.error(
                """
                Can't get max revision number for addressbook_id: {0}
                """.format(addressbook_id)
            )
            return None

        model_revision = models.AddressbookObjectRevision

        # use session here for func.max
        session = self._session()

        db_max_revision = (
            session.query(func.max(model_revision.id)
                          ).filter_by(id_addressbook=addressbook_id)
        ).scalar()

        if not db_max_revision:
            db_max_revision = 0

        return db_max_revision

    def get_modified_addressbook_objects(self, addressbook_id, revision_id):
        """
        Get modified addressbook objects with revision id greater than ``revision_id``.

        ``addressbook_id`` and ``revision_id`` must not be empty.
        """
        if addressbook_id is None or revision_id is None:
            self._logger.error(
                """
                Can't get modified addressbook objects for addressbook_id: {0}, revision_id: {1}
                """.format(addressbook_id,
                           revision_id)
            )
            return None

        model_revision = models.AddressbookObjectRevision
        record_modified = model_revision.where(
            id__gt=revision_id,
            id_addressbook=addressbook_id
        ).all()

        return (r.uri for r in record_modified)

    def delete_addressbook(self, addressbook_id):
        """Delete addressbook.

        ``addressbook_id`` must not be empty.
        """
        if not addressbook_id:
            self._logger.error(
                """
                Can't delete addressbook, id: {0}
                """.format(addressbook_id)
            )
            return

        model_addressbook = models.Addressbook
        db_addressbook = model_addressbook.where(id=addressbook_id).first()

        if model_addressbook is None:
            return

        if db_addressbook.type in [
            constant.ADDRESSBOOK_TYPE_DEFAULT,
            constant.ADDRESSBOOK_TYPE_SUGGESTED
        ]:
            raise RuntimeError(
                """
                Default addressbooks can't be deleted.
                addressbook info: {0}
                """.format(vars(db_addressbook))
            )

        self._logger.debug(
            "Deleting addressbook, id: {0}.".format(addressbook_id)
        )
        db_addressbook.delete()

    def delete_addressbook_object(self, principal_id, addressbook_id, uri):
        """Delete addressbook object.

        ``principal_id``, ``addressbook_id`` and ``uri`` must not be empty.
        """
        if not principal_id or not addressbook_id or not uri:
            self._logger.error(
                "Can't delete addressbook object for addressbook_id: {0}, uri: {1}"
                .format(addressbook_id,
                        uri)
            )
            return

        self._logger.debug(
            "Deleting addressbook object for addressbook_id: {0}, uri: {1}"
            .format(addressbook_id,
                    uri)
        )

        model_object = models.AddressbookObject
        model_revision = models.AddressbookObjectRevision

        db_object = model_object.where(
            id_addressbook=addressbook_id,
            uri=uri
        ).first()

        if db_object is None:
            return

        record_revision = {
            "id_principal": principal_id,
            "id_addressbook": addressbook_id,
            "uri": uri,
            "is_deleted": True
        }

        db_object.delete()
        model_revision.create(**record_revision)

    def get_addressbook_meta(self, addressbook_id):
        """Get metadata of addressbook.

        ``addressbook_id`` must not be empty.

        Return a dict with all properties.

        Addressbook props:
        {
                "D:displayname": "Address book",
                "CR:addressbook-description": "Addressbook created from web"
                "tag": "VADDRESSBOOK",
                "{http://inf-it.com/ns/ab/}addressbook-color": "#d1d353ff"
        }
        """
        if not addressbook_id:
            self._logger.error(
                "Can't get meta for addressbook_id: {0}".format(addressbook_id)
            )

        record = models.Addressbook.where(id=addressbook_id).first()

        if not record:
            return {}

        props = {
            constant.PROP_ADDRESSBOOK_DISPLAY_NAME: record.display_name,
            constant.PROP_ADDRESSBOOK_DESCRIPTION: record.description,
            constant.PROP_ADDRESSBOOK_COLOR: record.color,
            constant.PROP_TAG: constant.ADDRESSBOOK_TAG
        }
        return props

    def get_last_modified_time(self, id, collection_type):
        """Get last modified time.

        ``id`` must not be empty.

        ``collection_type`` must be in [constant.TYPE_PRINCIPAL, constant.TYPE_ADDRESSBOOK].
        """
        if not id or collection_type not in [
            constant.TYPE_PRINCIPAL,
            constant.TYPE_ADDRESSBOOK
        ]:
            self._logger.error(
                "Can't get last modified for id: {0}, collection_type: {1}"
                .format(id,
                        collection_type)
            )

        model_collection = get_collection_model(collection_type)
        collection = model_collection.where(id=id).first()

        if not collection:
            return None

        return utils.get_http_date(collection.modified_time)
